# -*- coding: utf-8 -*-
from . import constants
from django.db import models
from django.utils import timezone


class UrlRedirect(models.Model):

    url_source = models.TextField(unique=True)

    url_destination = models.TextField()

    date_initial_validity = models.DateTimeField(
        default=timezone.now(),
    )

    date_end_validity = models.DateTimeField(
        default=timezone.now(),
    )

    views = models.PositiveIntegerField(
        default=0,
        editable=False,
    )

    status = models.SmallIntegerField(
        choices=constants.STATUS_CHOICES,
        default=constants.STATUS_ACTIVE
    )

    created_at = models.DateTimeField(
        default=timezone.now(),
        editable=False,
    )

    updated_at = models.DateTimeField(
        default=timezone.now(),
        editable=False,
    )

    def __str__(self):
        return "({}) {}".format(self.status, self.url_source)

from signals import post_save_url


# -*- coding: utf-8 -*-
from ..models import UrlRedirect
from django.core.cache import cache
from django.utils import timezone
from django.db.models import F

def update_count_views_url(url_source):
    #validate url exist in cache
    cache_url_dict = cache.get('urls_redirect')
    if cache_url_dict and url_source in cache_url_dict:
        # get index
        obj = cache_url_dict[url_source]
        # validate if existe in range date
        if timezone.now()>= obj["date_initial_validity"] and \
           timezone.now() <= obj["date_end_validity"]:
            # update views in database
            UrlRedirect.objects.filter(url_source=url_source).update(views=F('views') + 1)
            return obj["url_destination"]

    return url_source


# -*- coding: utf-8 -*
from django.db.models.signals import post_save
from django.dispatch import receiver
from models import UrlRedirect
from django.core.cache import cache
from . import constants

@receiver(post_save, sender=UrlRedirect)
def post_save_url(sender, update_fields, instance=None, created=False,**kwargs):
    # get url_redirect (list of url)
    list_url_cache = cache.get('urls_redirect')
    # if exist delete url_redirect and url_redirect_dict
    if list_url_cache:
        cache.delete('urls_redirect')
    # get list_url_cache, url redirect active
    list_url_cache = UrlRedirect.objects.filter(status=constants.STATUS_ACTIVE).values("url_source","url_destination","date_initial_validity","date_end_validity","views")
    # create dictionary with unique index
    dict_url = {}
    for i in list_url_cache:
        dict_url[i["url_source"]]=i
    # save in cache list_url_cache and dict_url
    cache.set('urls_redirect', dict_url)
    pass


from django.contrib import admin
from models import UrlRedirect


class UrlAdmin(admin.ModelAdmin):
    list_filter = ['status']
    list_display = ['url_source', 'url_destination', 'views', 'status', 'date_initial_validity','date_end_validity']
    search_fields = ('url_source', 'url_destination',)


admin.site.register(UrlRedirect,UrlAdmin)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import datetime
from django.test import Client
from django.test import TestCase,override_settings
from ..models import UrlRedirect
from ..middleware import UrlRedirectMiddleware
from .. import constants
from ..actions import register

@override_settings(ROOT_URLCONF='urls.tests.urls')
@override_settings(DEBUG=True)
class ActionsTestCase(TestCase):
    def setUp(self):
        date_initial = datetime.strptime('Mon, 09 May 2016 08:30:15 GMT', '%a, %d %B %Y %H:%M:%S GMT')
        date_end = datetime.strptime('Mon, 15 May 2017 08:30:15 GMT', '%a, %d %B %Y %H:%M:%S GMT')
        date_initial3 = datetime.strptime('Mon, 09 May 2016 08:30:15 GMT', '%a, %d %B %Y %H:%M:%S GMT')
        date_end3 = datetime.strptime('Mon, 01 May 2017 08:30:15 GMT', '%a, %d %B %Y %H:%M:%S GMT')

        url_source1='/url_source1/'
        url_destination1='/url_destination1/'
        url_source2 = '/url_source2/'
        url_destination2 = '/url_destination2/'
        url_source3 = '/url_source3/'
        url_destination3 = '/url_destination3/'
        url1 = UrlRedirect(
            url_source=url_source1,
            url_destination=url_destination1,
            date_initial_validity=date_initial,
            date_end_validity=date_end
        )
        url2 = UrlRedirect(
            url_source=url_source2,
            url_destination=url_destination2,
            date_initial_validity=date_initial,
            date_end_validity=date_end
        )
        url3 = UrlRedirect(
            url_source=url_source3,
            url_destination=url_destination3,
            date_initial_validity=date_initial3,
            date_end_validity=date_end3
        )
        register.update_count_views_url(url_source1)
        url1.save()
        url2.save()
        register.update_count_views_url(url_source1)
        register.update_count_views_url(url_source1)
        url1=UrlRedirect.objects.filter(url_source=url_source1)[0]
        url1.status=constants.STATUS_ARCHIVED
        url1.save()
        register.update_count_views_url(url_source1)
        register.update_count_views_url(url_source3)
        url3.save()
        register.update_count_views_url(url_source3)
        register.update_count_views_url(url_source2)

        self.middleware = UrlRedirectMiddleware()
        self.client = Client()
        self.request = self.client.get('/url_source2/')

    def test_register_counter_url(self):
        url1 = UrlRedirect.objects.get(url_source="/url_source1/")
        url2 = UrlRedirect.objects.get(url_source="/url_source2/")
        url3 = UrlRedirect.objects.get(url_source="/url_source3/")

        self.assertEquals(url1.url_source, '/url_source1/')
        self.assertEquals(url1.views, 2)
        self.assertEquals(url2.url_source, '/url_source2/')
        self.assertEquals(url2.views, 1)
        self.assertEquals(url3.views, 0)

    def test_url_redirect_middleware(self):
        response = self.middleware.process_request(self.request._closable_objects[0])
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, '/url_destination2/')



from django.shortcuts import redirect
from actions import register


class UrlRedirectMiddleware():
    """
    This middleware lets you match a specific url and redirect the request to a
    new url.

    You keep a tuple of url regex pattern/url redirect tuples on your site
    settings, example:

    URL_REDIRECTS = (
        (r'www\.example\.com/hello/$', 'http://hello.example.com/'),
        (r'www\.example2\.com/$', 'http://www.example.com/example2/'),
    )

    """
    def process_request(self, request):
        host = request.META['PATH_INFO']
        redirect_url = register.update_count_views_url(host)
        if redirect_url == host:
            pass
        else:
            return redirect(redirect_url)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='UrlRedirect',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url_source', models.TextField(unique=True)),
                ('url_destination', models.TextField()),
                ('date_initial_validity', models.DateTimeField(default=django.utils.timezone.now)),
                ('date_end_validity', models.DateTimeField(default=django.utils.timezone.now)),
                ('views', models.PositiveIntegerField(default=0, editable=False)),
                ('status', models.SmallIntegerField(default=1, choices=[(1, b'Active'), (2, b'Archived'), (3, b'Deleted')])),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('updated_at', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]
